/* ------------------------------------------------------------------------
 *					       QuickDecision.c
 * ------------------------------------------------------------------------
 *
 */

#include <Pilot.h>
#include <Graffiti.h>

#include "QuickDecision.h"
#include "QuickDecisionConst.h"

DmOpenRef  theDB;

struct DecisionOptions  Options = { 1, 100, 50 };


unsigned  LimitedNumber( unsigned limit )
 {
  return ((unsigned long)limit * SysRandom( 0L ))/(sysRandomMax+1L);
 }


void    UpdateFieldString( Int f, CharPtr s )
 {
  FormPtr  frm;
  FieldPtr fld;

  frm = FrmGetActiveForm();

  fld = FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, f ) );
  FldSetTextPtr( fld, s );

  FldDrawField( fld );
 }


void    SetFieldNumber( Word frmNo, Word fldNo, Long l )
 {
  FormPtr	frm;
  FieldPtr	fld;
  Handle    hnd;
  CharPtr   cp;

  frm = FrmGetFormPtr( frmNo );
  fld = FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, fldNo ) );

  hnd = (Handle)MemHandleNew( 10 );
  if(!hnd)
    return;

  cp = MemHandleLock( (VoidHand)hnd );
  if(!cp)
    return;
  
  StrIToA( cp, l );
  MemPtrUnlock( cp );

  FldSetTextHandle( fld, hnd );
 }


void   UpdateRecord( VoidHand RecHnd, VoidPtr data, UInt size )
 {
  Ptr  RecPointer;

  RecPointer = MemHandleLock( RecHnd );
  DmWrite( RecPointer, 0, data, size );
  MemPtrUnlock( RecPointer );
 }


static   int  OpenDatabase( void )
 {
  UInt      index = 0;
  VoidHand 	RecHandle;
  Ptr       RecPointer;

  theDB = DmOpenDatabaseByTypeCreator( DATA_TYPE, CREATOR_ID, dmModeReadWrite );
  if(!theDB)
   {
    if(DmCreateDatabase( 0, "QuickDecisionPrefs", CREATOR_ID, DATA_TYPE, 
                         false ))
	 {
	  return 1;
	 }

    theDB = DmOpenDatabaseByTypeCreator( DATA_TYPE, CREATOR_ID, dmModeReadWrite );
	RecHandle  = DmNewRecord( theDB, &index, sizeof(Options) );
	if(!RecHandle)
	  return 1;

    UpdateRecord( RecHandle, &Options, sizeof(Options) );
	DmReleaseRecord( theDB, index, true );
   }
  else
   {
	RecHandle  = DmQueryRecord( theDB, index );
	if(!RecHandle)
	  return 1;

	RecPointer = MemHandleLock( RecHandle );
	MemMove( &Options, RecPointer, sizeof(Options) );
	MemPtrUnlock( RecPointer );
	DmReleaseRecord( theDB, index, true );
   }

  return 0;
 }


Word    StartApplication( void )
 {
  // Open databases or initialize system here.
  int  err=0;
  
  if(!(err = OpenDatabase()))
    FrmGotoForm( formMain );

  return err;
 }


void  CoinToss( void )
 {
  static char  szHeads[] = "Heads";
  static char  szTails[] = "Tails";

  UpdateFieldString( fldCoin, LimitedNumber( 2 ) ? szHeads : szTails );
 }


void  PickNumber( void )
 {
  static char  szNumber[20] = "";
  unsigned     rand = LimitedNumber(Options.High-Options.Low+1) + Options.Low;

  UpdateFieldString( fldPick, StrIToA( szNumber, (Long)rand ) );
 }



void  PercentLikely( void )
 {
  static char  szTrue[]  = "True";
  static char  szFalse[] = "False";

  UpdateFieldString( fldLikely, (LimitedNumber( 100 ) < Options.Percent) 
                                ? szTrue : szFalse );
 }


void  Oracle( void )
 {
  static char *Answers[24] =
   {
	"It is certain",
	"It is decidedly so",
	"So it is written",
    "Most likely", 
	"Outlook good",
	"Signs point to yes",
	"Without a doubt",
    "Yes",
	"You may rely on it",

	"Don't count on it",
	"My reply is no",
    "My sources say no",
	"No",
	"Not a chance",
	"Outlook not so good",
    "Very doubtful",
	"You must be joking",
	"The spirits say no",

	"Ask again later",
    "Cannot predict now",
	"Concentrate and ask again",
    "Reply hazy, try again",
	"The future is uncertain",
	"I have no answer at this time"
   };

  UpdateFieldString( fldOracle, Answers[LimitedNumber( sizeof(Answers)/sizeof(Answers[0]) )] );
 }



int  GetFieldNumber( Word frmNo, Word fldNo )
 {
  FormPtr	frm;
  FieldPtr	fld;
  Handle    hnd;
  CharPtr	cp;
  int       val;
  
  frm = FrmGetFormPtr( frmNo );
  fld = FrmGetObjectPtr( frm, FrmGetObjectIndex( frm, fldNo ) );

  hnd = FldGetTextHandle( fld );
  if(!hnd)
    return 0;

  cp = MemHandleLock( (VoidHand)hnd );

  if(!cp)
    return 0;
  
  val = StrAToI( cp );

  MemPtrUnlock( cp );

  return val;
 }



void SetRange( void )
 {
  Int      low;
  Int      high;
  CharPtr  cp;

  low  = GetFieldNumber( formRange, fldLow );
  high = GetFieldNumber( formRange, fldHigh );

  if(0 > low || low > sysRandomMax)
   {
    cp = "Low value must be between 1 and 32767.";
   }
  else if(0 > high || high > sysRandomMax)
   {
    cp = "High value must be between 1 and 32767.";
   }
  else if(low == high)
   {
    cp = "The high and low values may not be the same.";
   }
  else
   {
    if(low > high)
     {
      // if out of order, swap them
      Options.Low  = high;
      Options.High = low;
     }
	else
	 {
      Options.Low  = low;
      Options.High = high;
	 }

    FrmReturnToForm( formMain );

	return;
   }

  FrmCustomAlert( alertError, cp, 0, 0 );
 }


void  SetPercent( void )
 {
  Int percent = GetFieldNumber( formPercent, fldPercent );

  if(0 < percent && percent < 100)
   {
    Options.Percent = percent;
    FrmReturnToForm( formMain );
   }
  else
    FrmCustomAlert( alertError, "Percentage must be between 0 and 100%.", 
                    0, 0 );
 }

void  ReseedGenerator( void )
 {
  Long seed = (Long)GetFieldNumber( formReseed, fldSeedHi ) << 16 |
              (Long)GetFieldNumber( formReseed, fldSeedLo );

  if(seed)
   {
    SysRandom( seed );
    FrmReturnToForm( formMain );
   }
  else
    FrmCustomAlert( alertError, "Both seed values must not be 0.", 0, 0 );
 }



Boolean   ButtonHandler( int ctrlID )
 {
  switch(ctrlID)
   {
    case buttonCoin:
	  CoinToss();
	  return 1;

    case buttonPick:
      PickNumber();
	  return 1;

    case buttonLikely:
	  PercentLikely();
	  return 1;

    case buttonOracle:
      Oracle();
	  return 1;

    case buttonRange:
      SetRange();
	  return 1;

    case buttonPercent:
      SetPercent();
	  return 1;

    case buttonReseed:
      ReseedGenerator();
	  return 1;

    case buttonCancel:
      FrmReturnToForm( formMain );
	  return 1;

    default:
	  break;
   }

  return 0;
 }


Boolean   MenuHandler( int menuitem )
 {
  switch(menuitem)
   {
	case cmdRange:
      FrmPopupForm( formRange );
	  return 1;

	case cmdPercent:
      FrmPopupForm( formPercent );
	  return 1;

	case cmdReseed:
      FrmPopupForm( formReseed );
	  return 1;

	case cmdAbout:
	  FrmAlert( alertAbout );
	  return 1;

    default:
	  break;
   }

  return 0;
 }


Boolean   FormHandler( EventPtr event )
 {
  switch(event->eType)
   {
    case ctlSelectEvent:
	  // deal with buttons here
	  return ButtonHandler( event->data.ctlEnter.controlID );

    case frmOpenEvent:
	  // Handle form initialization here, then draw the form.
      FrmDrawForm( FrmGetActiveForm() );
      return 1;

    case menuEvent:
	  return MenuHandler( event->data.menu.itemID );

    default:
      break;
   }

  return 0;
 }



void    LoadForm( Word formID )
 {
  FormPtr  form;

  form = FrmInitForm( formID );
  FrmSetActiveForm( form );

  switch(formID)
   {
    case formRange:
	  SetFieldNumber( formID, fldHigh, Options.High );
	  SetFieldNumber( formID, fldLow, Options.Low );
	  break;

	case formPercent:
	  SetFieldNumber( formID, fldPercent, Options.Percent );
	  break;

    default:
	  break;
   }

  // If more than one form, load the correct handler.
  FrmSetEventHandler( form, FormHandler );
 }



void    EventLoop( void )
 {
  EventType  event;
  Word       err;

  do
   {
    EvtGetEvent( &event, evtWaitForever );

    if(SysHandleEvent( &event ))
      continue;

	if(MenuHandleEvent( (void *)0, &event, &err ))
	  continue;

    if(frmLoadEvent == event.eType)
	 {
      LoadForm( event.data.frmLoad.formID );
	  continue;
	 }

	FrmDispatchEvent( &event );
   } while(appStopEvent != event.eType);
 }



void   StopApplication( void )
 {
  UInt      index = 0;
  VoidHand 	RecHandle;
  Ptr       RecPointer;

  RecHandle  = DmGetRecord( theDB, index );
  if(RecHandle)
   {
    UpdateRecord( RecHandle, &Options, sizeof(Options) );
    DmReleaseRecord( theDB, index, true );
   }
  else
   {
    FrmCustomAlert( alertError, "Error Updating Preferences", 0, 0 );
   }

  DmCloseDatabase( theDB );

  // Close down app at this point
 }



DWord  PilotMain( Word cmd, Ptr cmdPBP, Word launchFlags )
{
  Word error;

  if (cmd == sysAppLaunchCmdNormalLaunch)  // Command test, could also use "if (!cmd)"
  {
    error = StartApplication();            // Application start code
    if(error) 
      return error;

    EventLoop();                           // Event loop

    StopApplication ();                    // Application stop code
  }
  return 0;
}
